@@ -1,23 +1,23 @@
 /*
  * packETH - ethernet packet generator
  * By Miha Jemec <jemcek@gmail.com>
- * Copyright 2003 Miha Jemec, Iskratel
+ * Copyright 2003-2018 Miha Jemec
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation, either version 3 of the License, or
+ (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * function.c - all routines except callbacks and routines for sending
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * 
  */
 
 #include <gtk/gtk.h>
@@ -46,16 +46,13 @@ struct pcaprec_hdr {
     guint32     orig_len;       /* actual length of packet */
 };
 
-/* Link-layer type; */
-static unsigned long pcap_link_type = 1;   /* Default is DLT-EN10MB */
-
 /* struct for clist */
 struct clist_hdr {
 	gint16 pnrb;	/* packet number */
 	gint32 time;	/* delay since previous packet */
 	gint16 plen; 	/* packet length */
-	char src[21];
-	char dst[21];
+	char src[40];
+	char dst[40];
 	gchar info[21];
 };
 	
@@ -65,10 +62,12 @@ typedef enum {
 	ETH_802_3,
 	ARP,
 	IPv4,
+	IPv6,
 	UDP,
 	TCP,
 	IGMP,
-	ICMP
+	ICMP,
+	ICMPv6
 } protocol_type;
 
 	
