@@ -1,24 +1,23 @@
 /*
  * packETH - ethernet packet generator
- * By Miha Jemec <m.jemec@iskratel.si>
- * Copyright 2003 Miha Jemec, Iskratel
+ * By Miha Jemec <jemcek@gmail.com>
+ * Copyright 2003-2014 Miha Jemec
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation, either version 3 of the License, or
+ (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * 
  */
+
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,9 +31,10 @@
 //#include "libnet/libnet-headers.h"
 //#include "headers.h"
 
-static GtkWidget *w1, *w2, *w3, *w4, *w5, *w6, *w7;
-char field[3100];
+static GtkWidget *w1, *w2, *w3, *w4, *w5, *w6, *w7, *w8;
+char field[31000];
 char temp[20];
+char temp6[40];
 char *ptrf;
 char *ptrt;
 int next_prot;
@@ -50,13 +50,10 @@ int load_gen_b_data(GtkButton *button, FILE *file_p) {
         char buffc[11][200];
 	int c, j, k;
 
-	w1 = lookup_widget(GTK_WIDGET (button), "radiobutton35");
-	w2 = lookup_widget(GTK_WIDGET (button), "radiobutton34");
 	w3 = lookup_widget(GTK_WIDGET (button), "optionmenu9");
 	w4 = lookup_widget(GTK_WIDGET (button), "entry109");
 	w5 = lookup_widget(GTK_WIDGET (button), "entry110");
 	w6 = lookup_widget(GTK_WIDGET(button), "checkbutton35");
-	w7 = lookup_widget(GTK_WIDGET(button), "checkbutton37");
 	/* we read the file ohh python, where are you... */
 	k = 0;
 	j = 0;
@@ -154,7 +151,7 @@ int load_gen_b_data(GtkButton *button, FILE *file_p) {
 int load_gen_s_data(GtkButton *button, FILE *file_p) {
 
 	long int buff4[5];
-        char buff[10];
+        char buff[100];
         char buffc[11][200];
 	int c, j, k;
 	char *ptr = NULL, *ptr2 = NULL;
@@ -350,7 +347,7 @@ int load_data(GtkButton *button, FILE *file_p, int whocalled, int howmanypackets
 	guint32 usecu = 0, usecb = 0;
 	double timediff = 0;
 	double timebeg= 0;
-	char pkt_temp[3100];
+	char pkt_temp[31000];
 	GtkWidget *clis;
 
 	/* first we read the pcap file header */
@@ -374,8 +371,6 @@ int load_data(GtkButton *button, FILE *file_p, int whocalled, int howmanypackets
 		/* if EOF, exit */
 		if (freads == 0)
 			return 1;
-
-		/* copy the 16 bytes into ph structure */
 		memcpy(&ph, pkt_temp, 16);
 
 		/* and the packet itself, but only up to the capture length */
@@ -396,14 +391,16 @@ int load_data(GtkButton *button, FILE *file_p, int whocalled, int howmanypackets
 	}
 
 	else { 
-	
+		GtkTreeModel *model;
+
 		clis = lookup_widget(GTK_WIDGET (button), "clist2");
-		gtk_clist_clear(GTK_CLIST(clis));
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (clis));
+		gtk_list_store_clear (GTK_LIST_STORE (model));
 	
 		for (j=0; j<howmanypackets; j++) {
 	
 			/* next the  pcap packet header */
-			fread(pkt_temp, sizeof(ph), 1, file_p);
+			freads = fread(pkt_temp, sizeof(ph), 1, file_p);
 
 			/* if EOF, exit */
 			if (freads == 0)
@@ -412,6 +409,11 @@ int load_data(GtkButton *button, FILE *file_p, int whocalled, int howmanypackets
 			/* copy the 16 bytes into ph structure */
 			memcpy(&ph, pkt_temp, 16);
 
+			if ((sizeof(ph)+ph.incl_len)>9200) {
+				error("Packets longer than 9200 bytes in pcap file");
+				continue;
+			}
+
 			/* and the packet itself, but only up to the capture length */
 			freads = fread(pkt_temp+sizeof(ph), ph.incl_len, 1, file_p);
 
@@ -426,8 +428,7 @@ int load_data(GtkButton *button, FILE *file_p, int whocalled, int howmanypackets
 		
 			/* we have to dissect the packet to get information for the list */
 			load_packet_disector(button, field, 2, &clh, ph.incl_len);			
-			//printf("tukaj3 %s\n", field);
-			
+
 			/* calculate the time information */
 			if (j==0) { 
 				timediff = 0;
@@ -442,30 +443,27 @@ int load_data(GtkButton *button, FILE *file_p, int whocalled, int howmanypackets
 								(double)((double)ph.ts_usec - (double)usecb)) / 1000000;
 			}
 
-			//printf("tukaj2 %s\n", field);
-
 			secu = ph.ts_sec;
 			usecu = ph.ts_usec;
 
-			/* insert a new row into clist */
-			load_gen_p_data(button, clis, field, &ph, j+1, &clh, timediff, timebeg);
+			/* insert a new row into model */
+			load_gen_p_data(button, model, field, &ph, j+1, &clh, timediff, timebeg);
 		}
 		if (j == howmanypackets) 
 			error("Only first 1000 packets loaded!\nTo change this modify #define on top of callbacks.c");
 	}
 
 	return 1;
-
 }
 	
 
 /* this one loads the parameters from file into notebook2 (Genp page) */
-int load_gen_p_data(GtkButton *button, GtkWidget *clis, char *fieldek, struct pcaprec_hdr *ph2, 
+int load_gen_p_data(GtkButton *button, GtkTreeModel *model, char *fieldek, struct pcaprec_hdr *ph2, 
 					int pkt_nr, struct clist_hdr *clptr, double timediff, double timebeg) {
-
+	GtkTreeIter iter;
 	gchar *datap[8];
-	gchar fieldp[7][21];
-	gchar field_p[1][3100];
+	gchar fieldp[7][41];
+	gchar field_p[1][31000];
 
 	datap[0]=&fieldp[0][0];
 	datap[1]=&fieldp[1][0];
@@ -486,8 +484,8 @@ int load_gen_p_data(GtkButton *button, GtkWidget *clis, char *fieldek, struct pc
 	else
 		g_snprintf(fieldp[3], 20, "%d !", (*ph2).incl_len);
 
-	g_snprintf(fieldp[4], 20, "%s", clptr->src);
-	g_snprintf(fieldp[5], 20, "%s", clptr->dst);
+	g_snprintf(fieldp[4], 40, "%s", clptr->src);
+	g_snprintf(fieldp[5], 40, "%s", clptr->dst);
 
 	switch (protokol) {
 		case ETH_II: {
@@ -506,6 +504,10 @@ int load_gen_p_data(GtkButton *button, GtkWidget *clis, char *fieldek, struct pc
 			g_snprintf(fieldp[6], 20, "IPv4");
 			break;
 		}
+		case IPv6: {
+			g_snprintf(fieldp[6], 20, "IPv6");
+			break;
+		}
 		case TCP: {
 			g_snprintf(fieldp[6], 20, "TCP");
 			break;
@@ -522,14 +524,27 @@ int load_gen_p_data(GtkButton *button, GtkWidget *clis, char *fieldek, struct pc
 			g_snprintf(fieldp[6], 20, "ICMP");
 			break;
 		}
+		case ICMPv6: {
+			g_snprintf(fieldp[6], 20, "ICMPv6");
+			break;
+		}
 	}
 
 	g_snprintf(field_p[0], 2*(32+(*ph2).incl_len), "%s", fieldek);
 		
-	gtk_clist_append(GTK_CLIST(clis), datap);
-	
-	return 1;
+	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
+	gtk_list_store_set(GTK_LIST_STORE(model), &iter,
+                       /*COL_NR*/0, datap[0],
+                       /*COL_TIME*/1, datap[1],
+                       /*COL_DELTA_TIME*/2, datap[2],
+                       /*COL_LENGTH*/3, datap[3],
+                       /*COL_SOURCE*/4, datap[4],
+                       /*COL_DESTINATION*/5, datap[5],
+                       /*COL_PROTOCOL*/6, datap[6],
+                       /*COL_INFO*/7, datap[7],
+                       -1);
 
+	return 1;
 }
 
 /* this routine was changed. it loads the packet into notebook2 (Builder page) or it checks the file containing the packets
@@ -595,7 +610,7 @@ int load_packet_disector(GtkButton *button, char *fieldek, int whocalled, struct
 	else 
 		memcpy(clptr->src, temp, 20);
 
-	/* next there is type or length field or 802.1q */
+	/* next there is type or length field or 802.1q or QinQ! */
 	i = char2x(ptrf)*256 + char2x(ptrf+2);
 
 	remain = remain - 14;
@@ -603,56 +618,31 @@ int load_packet_disector(GtkButton *button, char *fieldek, int whocalled, struct
 	/* in case of a vlan tag 0x8100 == 33024) */
 	w1 = lookup_widget(GTK_WIDGET(button), "bt_8021q");
 	w2 = lookup_widget(GTK_WIDGET(button), "frame6");
-	if (i == 33024) {
-		if (remain < 4) {
-			error("Can't load packet: Ethernet VLAN field is not long enough!");
-			return -1;
-		}
-		remain = remain -4;
 
-		ptrf = ptrf + 4;
-		*ptrt++ = '0';
-		*ptrt-- = *ptrf++;
-		i = char2x(ptrt);
-
-		if (whocalled == 1) {
-			w3 = lookup_widget(GTK_WIDGET(button), "L_optmenu2_bt");
-			w4 = lookup_widget(GTK_WIDGET(button), "checkbutton39");
-			w5 = lookup_widget(GTK_WIDGET(button), "checkbutton40");
-			w6 = lookup_widget(GTK_WIDGET(button), "L_vlan_id");
-			w7 = lookup_widget(GTK_WIDGET(button), "entry165");
+	if ((i == 33024) || (i==34984) || (i==37120) || (i==37376)) {
+		w3 = lookup_widget(GTK_WIDGET(button), "L_optmenu2_bt");
+		w4 = lookup_widget(GTK_WIDGET(button), "checkbutton39");
+		w5 = lookup_widget(GTK_WIDGET(button), "checkbutton40");
+		w6 = lookup_widget(GTK_WIDGET(button), "L_vlan_id");
+		w7 = lookup_widget(GTK_WIDGET(button), "entry165");
+		w8 = lookup_widget(GTK_WIDGET(button), "optionmenu21");
 		
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
-			gtk_widget_set_sensitive (w2, TRUE);
-		
-			gtk_option_menu_set_history (GTK_OPTION_MENU (w3), (i>>1));
-
-			if ( (i%2) == 0)
-				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w4), FALSE);
-			else
-				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w4), TRUE);
-
-			inspar(button, "L_vlan_id", ptrf, 3);
-		}
-		else
-			ptrf = ptrf+3;
-
-		i = char2x(ptrf)*256 + char2x(ptrf+2);
-
-		/* we support now vlan stacking, in case the protocol in once again 0x8100
-		 * we just add next 4 bytes in a QinQ field and switch the toggle button */
-		if (i == 33024) {
-			if (remain < 4) {
-				error("Can't load packet: QinQ VLAN field is not long enough!");
-				return -1;
-			}
-			remain = remain -4;
-
+	        //if we have 8100 after the next 4 bytes we do QinQ	
+		if ( ((char2x(ptrf+8)*256 + char2x(ptrf+10))==33024) && remain>=8) 
+		{
 			ptrf = ptrf + 4;
 			
 			if (whocalled == 1) {
 				gtk_widget_set_sensitive (w7, TRUE);
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w5), TRUE);
+				if (i==33024)
+					gtk_combo_box_set_active (GTK_COMBO_BOX (w8), 0);
+				else if (i==34984)
+					gtk_combo_box_set_active (GTK_COMBO_BOX (w8), 3);
+				else if (i==37120)
+					gtk_combo_box_set_active (GTK_COMBO_BOX (w8), 1);
+				else if (i==37376)
+					gtk_combo_box_set_active (GTK_COMBO_BOX (w8), 2);
 
 				inspar(button, "entry165", ptrf, 4);
 			}
@@ -666,11 +656,43 @@ int load_packet_disector(GtkButton *button, char *fieldek, int whocalled, struct
 				gtk_widget_set_sensitive (w7, FALSE);
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w5), FALSE);
 			}	
+
 		}
+
+		if (remain < 4) {
+			error("Can't load packet: Ethernet VLAN field is not long enough!");
+			return -1;
+		}
+		remain = remain -4;
+
+		ptrf = ptrf + 4;
+		*ptrt++ = '0';
+		*ptrt-- = *ptrf++;
+		i = char2x(ptrt);
+
+		if (whocalled == 1) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+			gtk_widget_set_sensitive (w2, TRUE);
+		
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w3), (i>>1));
+
+			if ( (i%2) == 0)
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w4), FALSE);
+			else
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w4), TRUE);
+
+			inspar(button, "L_vlan_id", ptrf, 3);
+		}
+		else
+			ptrf = ptrf+3;
+
+		i = char2x(ptrf)*256 + char2x(ptrf+2);
+
 	}
 	else {
 		if (whocalled == 1) {
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE);
+
 			gtk_widget_set_sensitive (w2, FALSE);
 		}	
 	}
@@ -791,19 +813,59 @@ int load_packet_disector(GtkButton *button, char *fieldek, int whocalled, struct
 
 	/* when ipv6 will be added, activate ipv6 button instead of userdef button */
 	else if (next_prot == 34525) {
-		//w1 = lookup_widget(GTK_WIDGET(button), "IPv6_rdbt");
-		//gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
-		w1 = lookup_widget(GTK_WIDGET(button), "usedef2_radibt");
+
+		/* ok, ipv6 should follow, so we call the routine for parsing ipv6 header. */
+                next_prot = ipv6_header(button, whocalled, clptr);
+                if (next_prot == -1)
+                        return -1;
+
+		w1 = lookup_widget(GTK_WIDGET(button), "IPv6_rdbt");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
 
-		ptrf = ptrf - 4;
-   		inspar(button, "L_ethtype", ptrf, 4);
-		
-		/* -2 means only llc without snap was used, so we don't insert the value in pid field */
-		if (next_prot != -2) {
-			ptrf = ptrf - 4;
-			inspar(button, "L_pid", ptrf, 4);
+                /* here we do the further parsing: tcp, udp, icmp, ...*/
+                if (next_prot == 58) {
+                        /* try to parse icmpv6 header */
+                        next_prot = icmpv6_header(button, whocalled);
+                        /* not ok, return an error */
+                        if (next_prot == -1)
+                                return -1;
+                        /* ok, lets activate the icmpv6 notebook */
+                        else {
+                                w1 = lookup_widget(GTK_WIDGET(button), "radiobutton69");
+                                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+                        }
+                }
+                else if (next_prot == 6) {
+                        /* try to parse tcp header */
+                        next_prot = tcp_header(button, whocalled);
+                        /* not ok, return an error */
+                        if (next_prot == -1)
+                                return -1;
+                        /* ok, lets activate the tcp notebook */
+                        else {
+                                w1 = lookup_widget(GTK_WIDGET(button), "radiobutton68");
+                                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+                        }
 		}
+		else if (next_prot == 17) {
+                        /* try to parse udp header */
+                        next_prot = udp_header(button, whocalled);
+                        /* not ok, return an error */
+                        if (next_prot == -1)
+                                return -1;
+                        /* ok, lets activate the udp notebook */
+                        else {
+                                w1 = lookup_widget(GTK_WIDGET(button), "radiobutton67");
+                                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+                        }
+                }
+                /* protocol we do not support yet; user defined window */
+                else {
+                        next_prot = usedef_insert(button, "text2", whocalled);
+                        w1 = lookup_widget(GTK_WIDGET(button), "radiobutton71");
+                        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+                }
+
 	}
 	/* anything else - user defined */
 	else {
@@ -950,7 +1012,7 @@ int arp_header(GtkButton *button, int whocalled) {
 int igmp_header(GtkButton *button, int whocalled) {
 
 	int x, x1;
-	char tmp[4];
+	char tmp[5];
 
 	if (whocalled==2) {
 		protokol = IGMP;
@@ -974,33 +1036,33 @@ int igmp_header(GtkButton *button, int whocalled) {
 	w2 = lookup_widget(GTK_WIDGET(button), "notebook8");
 	if (x == 17) {
 		if (remain > 4) {
-	        	gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 1);
-			gtk_notebook_set_page(GTK_NOTEBOOK(w2), 1);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 1);
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(w2), 1);
 			}
 		else	{
-	        	gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 0);
-			gtk_notebook_set_page(GTK_NOTEBOOK(w2), 0);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 0);
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(w2), 0);
 			}
 	}
 	else if (x == 18) {
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 2);
-		gtk_notebook_set_page(GTK_NOTEBOOK(w2), 0);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 2);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w2), 0);
 		}
 	else if (x == 22) {
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 3);
-		gtk_notebook_set_page(GTK_NOTEBOOK(w2), 0);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 3);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w2), 0);
 		}
 	else if (x == 34) {
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 4);
-		gtk_notebook_set_page(GTK_NOTEBOOK(w2), 2);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 4);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w2), 2);
 		}
 	else if (x == 23) {
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 5);
-		gtk_notebook_set_page(GTK_NOTEBOOK(w2), 0);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 5);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w2), 0);
 		}
 	else	{
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 6);
-		gtk_notebook_set_page(GTK_NOTEBOOK(w2), 0);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 6);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w2), 0);
 		}
 
 	inspar(button, "entry167", ptrf, 2);
@@ -1074,6 +1136,7 @@ int igmp_header(GtkButton *button, int whocalled) {
 int icmp_header(GtkButton *button, int whocalled) {
 
 	int x;
+	char tmp5[5];
 
 	if (whocalled==2) {
 		protokol = ICMP;
@@ -1095,19 +1158,19 @@ int icmp_header(GtkButton *button, int whocalled) {
 
 	w1 = lookup_widget(GTK_WIDGET(button), "optionmenu4");
 	if (x == 0)
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 0);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 0);
 	else if (x == 3) 
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 1);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 1);
 	else if (x == 8) 
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 2);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 2);
 	else
-	        gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 3);
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 3);
 
 
 	if (x == 0) { /* echo reply */
 		/* insert code, checksum, identifier and seq number and data if there is some */
 		w1 = lookup_widget(GTK_WIDGET(button), "notebook5");
-		gtk_notebook_set_page(GTK_NOTEBOOK(w1), 0);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w1), 0);
 		inspar(button, "entry62", ptrf, 2);
 		//inspar(button, "entry63", ptrf, 4);
 		ptrf = ptrf + 4;
@@ -1117,7 +1180,6 @@ int icmp_header(GtkButton *button, int whocalled) {
 		w2 = lookup_widget(GTK_WIDGET(button), "checkbutton16");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w2), TRUE);
 
-		inspar(button, "entry66", ptrf, remain * 2);
 		if (remain > 0) {
 			w1 = lookup_widget(GTK_WIDGET(button), "checkbutton17");
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
@@ -1126,11 +1188,14 @@ int icmp_header(GtkButton *button, int whocalled) {
 			w1 = lookup_widget(GTK_WIDGET(button), "checkbutton17");
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE);
 		}
+		inspar(button, "entry66", ptrf, 2);
+		sprintf(tmp5, "%d", remain);
+		inspar(button, "entry207", tmp5, 4);
 		
 	}
 	else if (x == 3) { /* destination unreacheable */
 		w1 = lookup_widget(GTK_WIDGET(button), "notebook5");
-		gtk_notebook_set_page(GTK_NOTEBOOK(w1), 2);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w1), 2);
 		/* which code? */
 		x = char2x(ptrf);
 		/* insert code */
@@ -1138,9 +1203,9 @@ int icmp_header(GtkButton *button, int whocalled) {
 
 		w1 = lookup_widget(GTK_WIDGET(button), "optionmenu5");
 		if ( (x >= 0) && (x <= 15) )
-			gtk_option_menu_set_history (GTK_OPTION_MENU (w1), x);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w1), x);
 		else
-			gtk_option_menu_set_history (GTK_OPTION_MENU (w1), 16);
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w1), 16);
 		
 		/* insert code, checksum, identifier and seq number and data if there is some */
 		//inspar(button, "entry59", ptrf, 4);
@@ -1150,7 +1215,6 @@ int icmp_header(GtkButton *button, int whocalled) {
 		w2 = lookup_widget(GTK_WIDGET(button), "checkbutton15");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w2), TRUE);
 
-		inspar(button, "entry61", ptrf, remain * 2);
 		if (remain > 0) {
 			w1 = lookup_widget(GTK_WIDGET(button), "checkbutton24");
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
@@ -1159,10 +1223,14 @@ int icmp_header(GtkButton *button, int whocalled) {
 			w1 = lookup_widget(GTK_WIDGET(button), "checkbutton24");
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE);
 		}
+
+		inspar(button, "entry66", ptrf, 2);
+		sprintf(tmp5, "%d", remain);
+		inspar(button, "entry210", tmp5, 4);
 	}
 	else if (x == 8) { /* echo request */
 		w1 = lookup_widget(GTK_WIDGET(button), "notebook5");
-		gtk_notebook_set_page(GTK_NOTEBOOK(w1), 5);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w1), 5);
 		/* insert code, checksum, identifier and seq number and data if there is some */
 		inspar(button, "entry74", ptrf, 2);
 		//inspar(button, "entry77", ptrf, 4);
@@ -1173,7 +1241,6 @@ int icmp_header(GtkButton *button, int whocalled) {
 		w2 = lookup_widget(GTK_WIDGET(button), "checkbutton20");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w2), TRUE);
 
-		inspar(button, "entry76", ptrf, remain * 2);
 		if (remain > 0) {
 			w1 = lookup_widget(GTK_WIDGET(button), "checkbutton19");
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
@@ -1182,11 +1249,14 @@ int icmp_header(GtkButton *button, int whocalled) {
 			w1 = lookup_widget(GTK_WIDGET(button), "checkbutton19");
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE);
 		}
+		inspar(button, "entry66", ptrf, 2);
+		sprintf(tmp5, "%d", remain);
+		inspar(button, "entry211", tmp5, 4);
 
 	}
 	else { /* all the rest */
 		w1 = lookup_widget(GTK_WIDGET(button), "notebook5");
-		gtk_notebook_set_page(GTK_NOTEBOOK(w1), 1);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(w1), 1);
 		/* insert code, checksum and data if there is some */
 		inspar(button, "entry157", ptrf, 2);
 		//inspar(button, "entry158", ptrf, 4);
@@ -1194,9 +1264,6 @@ int icmp_header(GtkButton *button, int whocalled) {
 		/* set checksum button on auto */
 		w2 = lookup_widget(GTK_WIDGET(button), "checkbutton38");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w2), TRUE);
-
-		inspar(button, "entry159", ptrf, (remain + 4) * 2);
-
 	}
 
 	return 1;
@@ -1206,10 +1273,10 @@ int icmp_header(GtkButton *button, int whocalled) {
 int usedef_insert(GtkButton *button, char *entry, int whocalled) {
 
 	int i, j;
-	char tmp[4600];
+	char tmp[31000];
 
-	if (whocalled == 1)
-		return 1;
+	//if (whocalled == 1)
+	//	return 1;
 
 	/* get access to buffer of the text field */
 	w2 = lookup_widget(GTK_WIDGET(button), entry);
@@ -1241,7 +1308,7 @@ int usedef_insert(GtkButton *button, char *entry, int whocalled) {
 int tcp_header(GtkButton *button, int whocalled) {
 
 	int x, i, j;
-	char tmp[4600], tmp2[3], ch;
+	char tmp[31000], tmp2[3], ch;
 
 	if (whocalled==2) {
 		protokol = TCP;
@@ -1373,7 +1440,7 @@ int tcp_header(GtkButton *button, int whocalled) {
 int udp_header(GtkButton *button, int whocalled) {
 
 	int i, j;
-	char tmp[4600];
+	char tmp[31000];
 
 	if (whocalled==2) {
 		protokol = UDP;
@@ -1629,7 +1696,7 @@ int ethernet_8023(GtkButton *button, int whocalled) {
 
 	//w2 = lookup_widget(GTK_WIDGET(button), "frame7");
 	//gtk_widget_set_sensitive (w2, TRUE);
-	//gtk_notebook_set_page(GTK_NOTEBOOK(w3), 1);
+	//gtk_notebook_set_current_page(GTK_NOTEBOOK(w3), 1);
 
 	w1 = lookup_widget(GTK_WIDGET(button), "entry5");
 	//inspar(button, "entry5", ptrf, 4);
@@ -1830,3 +1897,168 @@ void convert8field(char *to, char *from) {
 	memcpy(to+6, f1+0,1);
 	memcpy(to+7, f1+1,1);
 }
+
+int ipv6_header(GtkButton *button, int whocalled, struct clist_hdr *clptr ) {
+
+        //char tmp[5];
+        //int x; 
+	int prot, header_l=0;
+
+        if (whocalled==2) {
+                protokol = IPv6;
+        }
+
+        /* for standard header this is minimum length */
+        if (remain < 40) {
+                error("Can't load packet: IPv6 header field is not long enough!");
+                return -1;
+        }
+
+        if (whocalled==1) {
+                /* insert version */
+                inspar(button, "entry195", ptrf, 1);
+
+                /* insert traffic class */
+                inspar(button, "entry196", ptrf, 2);
+
+                /* insert tos */
+                inspar(button, "entry197", ptrf, 5);
+
+                /* insert total length */
+                w1 = lookup_widget(GTK_WIDGET(button), "checkbutton43");
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+                //insint(button, "entry29", ptrf, 4);
+        }
+        else
+        	ptrf = ptrf + 8;
+
+       	ptrf = ptrf + 4;
+
+
+	prot = char2x(ptrf);
+
+        if (whocalled==1) {
+                /* insert next header */
+                inspar(button, "entry199", ptrf, 2);
+        }
+        else
+                ptrf = ptrf+2;
+
+	 if (whocalled==1) {
+                /* insert hop limit */
+                insint(button, "entry200", ptrf, 2);
+	}
+        else
+                ptrf = ptrf+2;
+
+	/*insert source ip */
+        ptrt = temp6;
+        memset(temp6, 0, 40);
+
+        if (whocalled==1)
+                w1 = lookup_widget(GTK_WIDGET(button), "entry201");
+
+        for (i=1; i<8; i++) {
+		strncpy(ptrt, ptrf, 4);
+		ptrt = ptrt + 4;
+                strcat(ptrt, ":");
+		ptrt++;
+		ptrf = ptrf + 4;
+        }
+	strncpy(ptrt, ptrf, 4);
+	ptrf = ptrf + 4;
+
+        if (whocalled==1)
+                gtk_entry_set_text(GTK_ENTRY(w1), temp6);
+        else
+                memcpy(clptr->src, temp6, 40);
+
+	/*insert destination ip */
+        ptrt = temp6;
+        memset(temp6, 0, 40);
+        if (whocalled==1)
+                w1 = lookup_widget(GTK_WIDGET(button), "entry202");
+
+        for (i=1; i<8; i++) {
+		strncpy(ptrt, ptrf, 4);
+		ptrt = ptrt + 4;
+                strcat(ptrt, ":");
+		ptrt++;
+		ptrf = ptrf + 4;
+        }
+	strncpy(ptrt, ptrf, 4);
+	ptrf = ptrf + 4;
+
+        if (whocalled==1)
+                gtk_entry_set_text(GTK_ENTRY(w1), temp6);
+        else
+                memcpy(clptr->dst, temp6, 40);
+
+	//extension header
+	while ( (prot==0) || (prot==43) || (prot==44) || (prot==51) || (prot==50) || (prot==60) ) {
+		prot = char2x(ptrf);
+		header_l = retint2(ptrf+2, 2);
+		inspar(button, "entry203", ptrf, header_l);
+
+	}
+
+	remain = remain - 40 - header_l;
+
+        return prot;
+}
+
+int icmpv6_header(GtkButton *button, int whocalled) {
+
+	//char tmp5[5];
+
+	if (whocalled==2) {
+                protokol = ICMPv6;
+                return 1;
+        }
+	
+	/* for standard header this is minimum length */
+        if (remain < 4) {
+                error("Can't load packet: Packet length shorter than min ICMPv6 header length!");
+                return -1;
+        }
+
+	remain = remain -4;
+
+	inspar(button, "entry215", ptrf, 2);
+
+	inspar(button, "entry216", ptrf, 2);
+
+	w1 = lookup_widget(GTK_WIDGET(button), "checkbutton48");
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+        ptrf = ptrf + 4;
+    
+    //if (remain >=511)	{
+	//	inspar(button, "entry214", ptrf, 1024);
+	//	//remain = remain -4;
+	//}
+	//else if (remain > 0) {
+	//	inspar(button, "entry214", ptrf, remain*2);
+	//}
+	if (remain > 0) {
+		inspar(button, "entry214", ptrf, remain*2);
+	}
+	else
+		return 1;
+	
+	/*if (remain > 0) {
+                w1 = lookup_widget(GTK_WIDGET(button), "checkbutton47");
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
+         	inspar(button, "entry212", ptrf, 2);
+         	sprintf(tmp5, "%d", remain);
+         	inspar(button, "entry213", tmp5, 4);
+         }
+         else {
+                w1 = lookup_widget(GTK_WIDGET(button), "checkbutton47");
+                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE);
+         } */
+
+
+	return 1;
+
+}
+
